#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

#region Global Enums

public enum DMAType {EMA,HMA,WMA,SMA}

#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("2 Moving Averages Colors")]
    public class DoubleMA : Indicator
    {
        #region Variables
		
		private IDataSeries MA1Series;
		private IDataSeries MA2Series; 
		
		// Options
		private bool			showPlot			= true;
		private bool			colorBars			= false;
		private bool 			colorzone   		= false;
		private bool 			arrows 				= false;
		private bool			soundAlert			= false;
		
		// MA Plots
		private 				DMAType maType1		= DMAType.EMA;
		private 				DMAType maType2		= DMAType.WMA;
		private int 			maPeriod1			= 20;
		private int 			maPeriod2			= 50;
		private Color 			maColorUp			= Color.RoyalBlue;
		private Color 			maColorDown			= Color.Crimson;
		private Color 			maColorUp1			= Color.CornflowerBlue;
		private Color 			maColorDown1		= Color.Red;
		private int				plot0Width			= 2;
		private PlotStyle		plot0Style			= PlotStyle.Line;
		private DashStyle		dash0Style			= DashStyle.Solid;
		private int				plot1Width			= 2;
		private PlotStyle		plot1Style			= PlotStyle.Line;
		private DashStyle		dash1Style			= DashStyle.Solid;
		
		// Color Bars
		private bool			candles				= true;
		private int				opacity				= 1;
		private int				alpha				= 0;
		private Color 			barColorUp			= Color.CornflowerBlue;
		private Color 			barColorDown		= Color.Red;
		private Color 			candleOutlineUp		= Color.Navy;
		private Color 			candleOutlineDown	= Color.Firebrick;
		
		// Color Zones
		private int 			zopacity			= 1;
		private Color 			upColor				= Color.RoyalBlue;
		private Color 			downColor			= Color.Crimson;
		
		// Arrows
		private int 			arrowOffset 		= 4;
		private Color 			arrowColorUp		= Color.Navy;
		private Color			arrowColorDown		= Color.Firebrick;
		
		// Sound Alerts
		private string			upFile				= "Alert4.wav";
		private string			downFile			= "Alert3.wav";
		private int				rearmTime			= 30;
				
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Gray, PlotStyle.Line, "Fast"));
            Add(new Plot(Color.Gray, PlotStyle.Line, "Slow"));
            
			Overlay				= true;
            PlotsConfigurable	= false;
		}
		
		/// <summary>
		/// </summary>
		protected override void OnStartUp()
		{
			Plots[0].Pen.Width 		= plot0Width;
			Plots[1].Pen.Width 		= plot1Width;
			Plots[0].PlotStyle		= plot0Style;
			Plots[1].PlotStyle		= plot1Style;
			Plots[0].Pen.DashStyle 	= dash0Style;
			Plots[1].Pen.DashStyle 	= dash1Style;
			
			if (showPlot)
			{
				Plots[0].Pen.Color = Color.Gray;
				Plots[1].Pen.Color = Color.Gray;
			}
			else
			{
				Plots[0].Pen.Color = Color.Transparent;
				Plots[1].Pen.Color = Color.Transparent;
			}
			
			if (ChartControl != null && ChartControl.ChartStyleType == ChartStyleType.CandleStick)
			{
				candles				= true;
				alpha 				= 25*opacity;
			}
			
			switch(maType1)
			{
				case DMAType.EMA:
					MA1Series=EMA(maPeriod1);
					break;					
				case DMAType.HMA:
					MA1Series=HMA(maPeriod1);
					break;					
				case DMAType.WMA:
					MA1Series=WMA(maPeriod1);
					break;					
				case DMAType.SMA:
					MA1Series=SMA(maPeriod1);
					break;
			}
			
			switch(maType2)
			{
				case DMAType.EMA:
					MA2Series=EMA(maPeriod2);
					break;					
				case DMAType.HMA:
					MA2Series=HMA(maPeriod2);
					break;					
				case DMAType.WMA:
					MA2Series=WMA(maPeriod2);
					break;					
				case DMAType.SMA:
					MA2Series=SMA(maPeriod2);
					break;
			}
		}

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
          	if (CurrentBar < maPeriod1 + maPeriod2)
				return;
			
			Fast.Set(MA1Series[0]);
         	Slow.Set(MA2Series[0]);
			
			if (Values[0][0] > Values[1][0])
				{
					PlotColors[0][0] = maColorUp;
					PlotColors[1][0] = maColorUp1;
				}
				else if (Values[0][0] < Values[1][0])
				{
					PlotColors[0][0] = maColorDown;
					PlotColors[1][0] = maColorDown1;
				}
				else
				{
					PlotColors[0][0] = PlotColors[0][1];
					PlotColors[1][0] = PlotColors[1][1];
				}
						
			if (ColorBars) 
			{
				if (Values[0][0] > Values[1][0])
				{
					BarColor = barColorUp;
					CandleOutlineColor = candleOutlineUp;
				}
				else if (Values[0][0] < Values[1][0])
				{
					BarColor = barColorDown;
					CandleOutlineColor = candleOutlineDown;
				}
				else
				{
					CandleOutlineColor = CandleOutlineColorSeries[1];
					BarColor = BarColorSeries[1];
				}	
				
				if (candles && Close[0] > Open[0])
						BarColor = Color.FromArgb(alpha, BarColor);
			}
				
			if (Colorzone)
			{
				if (Values[0][0] > Values[1][0])
				{
					DrawRegion("MABands"+CurrentBar,1, 0, MA1Series, MA2Series, Color.Transparent, UpColor, Zopacity);
				}
				else 
				{
					DrawRegion("MABands"+CurrentBar,1, 0, MA1Series, MA2Series, Color.Transparent, DownColor, Zopacity);
				}
			}
				
			if (Arrows)
			{
				if (CrossAbove(Fast,Slow,1))
					DrawArrowUp("arrow"+CurrentBar, true, 0, Low[0]-arrowOffset*TickSize, arrowColorUp);
					
				if (CrossBelow(Fast,Slow,1))
					DrawArrowDown("arrow"+CurrentBar, true, 0, High[0]+arrowOffset*TickSize, arrowColorDown);
			}
				
			if (SoundAlert)
			{
				if (CrossAbove(Fast,Slow,1))
				{
					try
					{
						Alert("CrossAbove",NinjaTrader.Cbi.Priority.Medium,"CrossAbove",upFile,rearmTime,Color.Navy,Color.Magenta);
					}
						catch {}
				}
				if (CrossBelow(Fast,Slow,1))
				{
					try
					{
						Alert("CrossBelow",NinjaTrader.Cbi.Priority.Medium,"CrossBelow",downFile,rearmTime,Color.Navy,Color.Magenta);
					}
						catch {}
				}
			}
		}

        #region Properties
		
		// Parameters
	    [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fast
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Slow
        {
            get { return Values[1]; }
        }
		
		// Options
		[Description("Show plot of the ADXVMA average")]
        [Category("1.Options")]
		[Gui.Design.DisplayName ("1.Show MA Lines")]
        public bool ShowPlot
        {
            get { return showPlot; }
            set { showPlot = value; }
        }
				
		[Description("Color the bars?")]
        [Category("1.Options")]
        [Gui.Design.DisplayName("2.Color Bars")]
        public bool ColorBars
        {
            get { return colorBars; }
            set {colorBars = value; }
        }
		
		[Description("Color Zone?")]  
		[Category("1.Options")]
		[Gui.Design.DisplayNameAttribute("3.Color Zone")]
		public bool Colorzone
		{
			get { return colorzone; }
			set { colorzone = value; }
		}
		
		[Description("Show Arrows?")]
        [Category("1.Options")]
		[Gui.Design.DisplayName("4.Show Arrows")]
        public bool Arrows
        {
            get { return arrows; }
            set {arrows = value; }
        }
		
		[Description("Sound Alerts?")]
        [Category("1.Options")]
		[Gui.Design.DisplayName("5.Sound Alert")]
        public bool SoundAlert
        {
            get { return soundAlert; }
            set { soundAlert = value; }
        }
						
		//MA Fast
		[Description("Moving Average Type 1")]
        [Category("2.MA Fast")]
		[Gui.Design.DisplayNameAttribute("1.MA Type Fast ")]
        public  DMAType MA1Type
        {
            get { return maType1; }
            set { maType1 = value; }
        }
		
		[Description("MA Period 1")]
        [Category("2.MA Fast")]
		[Gui.Design.DisplayNameAttribute("2.MA Period Fast ")]
        public int MaPeriod1
        {
            get { return maPeriod1; }
            set { maPeriod1 = Math.Max(1, value); }
        }
				
		[XmlIgnore()]
        [Description("Up Color for MA")]
        [Category("2.MA Fast")]
        [Gui.Design.DisplayNameAttribute("3.MA Color Up Fast")]
        public Color MAColorUp
        {
            get { return maColorUp; }
            set { maColorUp = value; }
        }
		
        [Browsable(false)]
        public string maColorUpSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(maColorUp); }
            set { maColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
        [XmlIgnore()]
        [Description(" Down Color for MA")]
        [Category("2.MA Fast")]
        [Gui.Design.DisplayNameAttribute("4.MA Color Down Fast")]
        public Color MAColorDown
        {
            get { return maColorDown; }
            set { maColorDown = value; }
        }
		
        [Browsable(false)]
        public string maColorDownSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(maColorDown); }
            set { maColorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
		/// <summary>
		/// </summary>
		[Description("Width for MA Line.")]
		[Category("2.MA Fast")]
		[Gui.Design.DisplayNameAttribute("5.Line Width MA Fast")]
		public int Plot0Width
		{
			get { return plot0Width; }
			set { plot0Width = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("DashStyle for MA Line")]
		[Category("2.MA Fast")]
		[Gui.Design.DisplayNameAttribute("6.Plot Style MA Fast")]
		public PlotStyle Plot0Style
		{
			get { return plot0Style; }
			set { plot0Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for MA Line")]
		[Category("2.MA Fast")]
		[Gui.Design.DisplayNameAttribute("7.Dash Style MA Fast")]
		public DashStyle Dash0Style
		{
			get { return dash0Style; }
			set { dash0Style = value; }
		}
		
		//MA Slow
		[Description("Moving Average Type 2")]
        [Category("3.MA Slow")]
		[Gui.Design.DisplayNameAttribute("1.MA Type Slow ")]
        public  DMAType MA2Type
        {
            get { return maType2; }
            set { maType2 = value; }
        }
			
		[Description("MA Period 2")]
        [Category("3.MA Slow")]
		[Gui.Design.DisplayNameAttribute("2.MA Period Slow ")]
        public int MaPeriod2
        {
            get { return maPeriod2; }
            set { maPeriod2 = Math.Max(1, value); }
        }
				
		[XmlIgnore()]
        [Description("Up Color for MA")]
        [Category("3.MA Slow")]
        [Gui.Design.DisplayNameAttribute("3.MA Color Up Slow")]
        public Color MAColorUp1
        {
            get { return maColorUp1; }
            set { maColorUp1 = value; }
        }
		
        [Browsable(false)]
        public string maColorUp1Serialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(maColorUp1); }
            set { maColorUp1 = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
        [XmlIgnore()]
        [Description(" Down Color for MA")]
        [Category("3.MA Slow")]
        [Gui.Design.DisplayNameAttribute("4.MA Color Down Slow")]
        public Color MAColorDown1
        {
            get { return maColorDown1; }
            set { maColorDown1 = value; }
        }
		
        [Browsable(false)]
        public string maColorDown1Serialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(maColorDown1); }
            set { maColorDown1 = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
		/// <summary>
		/// </summary>
		[Description("Width for MA Line.")]
		[Category("3.MA Slow")]
		[Gui.Design.DisplayNameAttribute("5.Line Width MA Slow")]
		public int Plot1Width
		{
			get { return plot1Width; }
			set { plot1Width = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("DashStyle for MA Line")]
		[Category("3.MA Slow")]
		[Gui.Design.DisplayNameAttribute("6.Plot Style MA Slow")]
		public PlotStyle Plot1Style
		{
			get { return plot1Style; }
			set { plot1Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for MA Line")]
		[Category("3.MA Slow")]
		[Gui.Design.DisplayNameAttribute("7.Dash Style MA Slow")]
		public DashStyle Dash1Style
		{
			get { return dash1Style; }
			set { dash1Style = value; }
		}
		
		// Color Bars
		[XmlIgnore]
        [Description("Color of Bar Up")]
        [Category("4.Bars")]
        [Gui.Design.DisplayNameAttribute("1.Bar Color Up")]
        public Color BarColorUp
        {
            get { return barColorUp; }
            set { barColorUp = value; }
        }

        [Browsable(false)]
        public string BarColorUpSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(barColorUp); }
            set { barColorUp = Gui.Design.SerializableColor.FromString(value); }
        }
		
		[XmlIgnore]
        [Description("Color of Bar Down")]
        [Category("4.Bars")]
        [Gui.Design.DisplayNameAttribute("2.Bar Color Down")]
        public Color BarColorDown
        {
            get { return barColorDown; }
            set { barColorDown = value; }
        }

        [Browsable(false)]
        public string BarColorDownSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(barColorDown); }
            set { barColorDown = Gui.Design.SerializableColor.FromString(value); }
        }
		
		[XmlIgnore]
        [Description("Color of CandleOutlineUp.")]
        [Category("4.Bars")]
        [Gui.Design.DisplayNameAttribute("3.Candle Outline Up")]
        public Color CandleOutlineUp
        {
            get { return candleOutlineUp; }
            set { candleOutlineUp = value; }
        }

        [Browsable(false)]
        public string CandleOutlineUpSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(candleOutlineUp); }
            set { candleOutlineUp = Gui.Design.SerializableColor.FromString(value); }
        }
				
		[XmlIgnore]
        [Description("Color of CandleOutlineDown.")]
        [Category("4.Bars")]
        [Gui.Design.DisplayNameAttribute("4.Candle Outline Down")]
        public Color CandleOutlineDown
        {
            get { return candleOutlineDown; }
            set { candleOutlineDown = value; }
        }

        [Browsable(false)]
        public string CandleOutlineDownSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(candleOutlineDown); }
            set { candleOutlineDown = Gui.Design.SerializableColor.FromString(value); }
        }
		
		[Description("Opacity for Up Bars")]
		[Category("4.Bars")]
		[Gui.Design.DisplayNameAttribute("5.Opacity Up Bar")]
		public int Opacity
		{
			get { return opacity; }
			set { opacity = value; }
		}
		
		// Color Zones
		[Description("Zone Opacity")]
		[Category("5.Zone Coloring")]
		[Gui.Design.DisplayNameAttribute("1.Zone Opacity")]
		public int Zopacity
		{
			get { return zopacity; }
			set { zopacity = value; }
		}
				
		[XmlIgnore()]
        [Description("Color for Up Trend")]
        [Category("5.Zone Coloring")]
		[Gui.Design.DisplayNameAttribute("2.Zone Color Up")]
        public Color UpColor
        {
            get { return upColor; }
            set { upColor = value; }
        }
		
		[Browsable(false)]
		public string UpColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(UpColor); }
			set { UpColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("Color for Down Trend")]
        [Category("5.Zone Coloring")]
		[Gui.Design.DisplayNameAttribute("3.Zone Color Down")]
        public Color DownColor
        {
            get { return downColor; }
            set { downColor = value; }
        }
		
		[Browsable(false)]
		public string DownColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(DownColor); }
			set { DownColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		// Arrows
		[Description("Offset for Arrows")]
        [Category("6.Arrows")]
		[Gui.Design.DisplayName("1.Arrow Offset")]
        public int ArrowOffset
        {
            get { return arrowOffset; }
            set {arrowOffset = Math.Max(1, value); }
        }
		
		[XmlIgnore()]
        [Description("Up Color for Arrows")]
        [Category("6.Arrows")]
        [Gui.Design.DisplayNameAttribute("2.Arrow Color Up")]
        public Color ArrowColorUp
        {
            get { return arrowColorUp; }
            set { arrowColorUp = value; }
        }
		
        [Browsable(false)]
        public string arrowColorUpSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(arrowColorUp); }
            set { arrowColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
		[XmlIgnore()]
        [Description("Down Color for Arrows")]
        [Category("6.Arrows")]
        [Gui.Design.DisplayNameAttribute("3.Arrow Color Down")]
        public Color ArrowColorDown
        {
            get { return arrowColorDown; }
            set { arrowColorDown = value; }
        }
		
        [Browsable(false)]
        public string arrowColorDownSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(arrowColorDown); }
            set { arrowColorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
		// Sound Alerts
		[Description("Sound File for Cross Above")]
		[Category("7.Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("1.Cross Above File")]
		public string UpFile
		{
		get { return upFile; }
		set { upFile = value; }
		}
		
		[Description("Sound File for Cross Below")]
		[Category("7.Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("2.Cross Below File")]
		public string DownFile
		{
		get { return downFile; }
		set { downFile = value; }
		}
		
		[Description("Rearm time for alert in seconds")]
		[Category("7.Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("3.Rearm Time (sec)")]
		public int RearmTime
		{
		get { return rearmTime; }
		set { rearmTime = value; }
		}
				
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DoubleMA[] cacheDoubleMA = null;

        private static DoubleMA checkDoubleMA = new DoubleMA();

        /// <summary>
        /// 2 Moving Averages Colors
        /// </summary>
        /// <returns></returns>
        public DoubleMA DoubleMA()
        {
            return DoubleMA(Input);
        }

        /// <summary>
        /// 2 Moving Averages Colors
        /// </summary>
        /// <returns></returns>
        public DoubleMA DoubleMA(Data.IDataSeries input)
        {
            if (cacheDoubleMA != null)
                for (int idx = 0; idx < cacheDoubleMA.Length; idx++)
                    if (cacheDoubleMA[idx].EqualsInput(input))
                        return cacheDoubleMA[idx];

            lock (checkDoubleMA)
            {
                if (cacheDoubleMA != null)
                    for (int idx = 0; idx < cacheDoubleMA.Length; idx++)
                        if (cacheDoubleMA[idx].EqualsInput(input))
                            return cacheDoubleMA[idx];

                DoubleMA indicator = new DoubleMA();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                DoubleMA[] tmp = new DoubleMA[cacheDoubleMA == null ? 1 : cacheDoubleMA.Length + 1];
                if (cacheDoubleMA != null)
                    cacheDoubleMA.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheDoubleMA = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// 2 Moving Averages Colors
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DoubleMA DoubleMA()
        {
            return _indicator.DoubleMA(Input);
        }

        /// <summary>
        /// 2 Moving Averages Colors
        /// </summary>
        /// <returns></returns>
        public Indicator.DoubleMA DoubleMA(Data.IDataSeries input)
        {
            return _indicator.DoubleMA(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// 2 Moving Averages Colors
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DoubleMA DoubleMA()
        {
            return _indicator.DoubleMA(Input);
        }

        /// <summary>
        /// 2 Moving Averages Colors
        /// </summary>
        /// <returns></returns>
        public Indicator.DoubleMA DoubleMA(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DoubleMA(input);
        }
    }
}
#endregion
